/*
 *  @(#) Main.java 1.2 25-jun-99
 *
 *  Copyright by <ADRESS>Thomas Holland, innot@frankfurt.netsurf.de</ADRESS>
 *  This is part of the <I>presdec</I> application.
 *  For detailed information read the <TT>README</TT> file supplied with this application.
 *
 */


package presdec;

import java.awt.*;		// for the file selector
import java.io.*;
import java.util.Hashtable;

import presdec.PrcFile;
import presdec.resource.bitmap.*;

/**
 * 	presdec
 *
 *	The <TT>main</TT> method for the presdec application, packed into a seperate class
 *	for code readability.
 *
 *
 *	@author Thomas Holland
 *	@version 1.2  25-jun-99		Added new option -d for dumping resources in a human readable way
 *	@version 1.1  20-jun-99		Added support for bmp converter
 *	@version 1.0  30-sep-98
 *
 */
public class Main {

// Data members ----------------------------------------------------------


// End of data members ---------------------------------------------------

// Main method -----------------------------------------------------------

	/**
	 *	The main method of the presdec application. It parses the options passed
	 *	from the command line, checks everything, builds a <code>PrcFile</code> Object
	 *	and then calls some <code>PrcFile</code> methods as required by the
	 *	command line options to generate the output.
	 *
	 *	@param	args	Command line arguments. See README for more information.
	 *
	 *	@see presdec.PrcFile
	 *
	 */
	public static void main( String[] args ) {
	
		boolean verbose = false;

		Hashtable options = new Hashtable(10);
		 
	    	File prcfile;
		PrcFile prc = null;

		if ( args.length > 0) { // Do we have any command line arguments?
			// Step thru all options
			int i=0;
			char c=' ';
			while(i<args.length) {
				c = args[i].charAt(0); // Does the argument start with a "-"?
				if (c == (char)'-') { // one or more options
					for(int n=1; n<args[i].length(); n++) { // step thru the chars
						c = args[i].charAt(n);
						if(c == 'h') {
							usage();
							System.exit(0);
						}
						if(c == 'v') {
							options.put("verbose", new Boolean(true));
							verbose = true;
							continue;
						}
						if(c == 'V') {
							options.put("structure", new Boolean(true));
							break;
						}
						if(c == 'a') {
							options.put("all", new Boolean(true));
							continue;
						}
						if(c == 'A') {
							options.put("ALL", new Boolean(true));
							continue;
						}
						if(c == 'b') {
							if(options.containsKey("binary")) {
								fail("Options -b and -B may not be used together");
							}
							options.put("binary", new Boolean(false));
							continue;
						}
						if(c == 'B') {
							if(options.containsKey("binary")) {
								fail("Options -B and -b may not be used together");
							}
							options.put("binary", new Boolean(true));
							continue;
						}
						if(c == 'd') {
							options.put("dumpres", new Boolean(true));
							continue;
						}
						if(c == 's') {
							options.put("sort", new Boolean(true));
							continue;
						}
						if(c == 'i') {
							options.put("include", new File(args[++i]));
							break;
						}
						if(c == 'm') {
							options.put("makefile", new File(args[++i]));
							break;
						}
						if(c == 'o') {
							options.put("output", new File(args[++i]));
							break;
						}
						if(c == 'O') {
							options.put("NOPRC", new Boolean(true));
							continue;
						}
						if(c == 'p') {
							options.put("path", new File(args[++i]));
							break;
						}
						if(c == 't') { /** @(#) Converters */
							if(args[++i].equalsIgnoreCase("pbmascii")) {
								options.put("bitmapconv", new PbmAsciiConverter());
							}
							else if(args[i].equalsIgnoreCase("ascii")) {
								options.put("bitmapconv", new AsciiConverter());
							}
							else if(args[i].equalsIgnoreCase("bmp")) {
								options.put("bitmapconv", new BmpConverter());
							}
							else if(args[i].equalsIgnoreCase("none")) {
								options.put("bitmapconv", ""); // put something that is no Converter
							}
							else {
								fail("Illegal or unsupported bitmap conversion option.\n" +
								     "Supported formats: pbmascii, ascii and none");
							}
							break;
						}
						// default
						fail("presdec: unknown option -" + c);
					} // for
				} // if option
				else {// otherwise it is a filename
					if(options.containsKey("prcfile")) {	// We already have a filename
						fail("Too many prc filenames");
					}
					options.put("prcfile", new File(args[i]));
				}
				i++;
			} // while
		} // if args


		////////////////////////////////////////////////////////
		// Fill in the default options values if non supplied //
		////////////////////////////////////////////////////////

		//
		// If no prc file is supplied on the command line, then we pop up a file requester
		// 
		if(!options.containsKey("prcfile")) {
			FileDialog fd = new FileDialog( new Frame(), "Load PalmPilot Resource (prc) File" /*, FileDialog.LOAD*/ );
			// fd.setFilenameFilter(new PrcFilter()); // Does not work yet!

			fd.show();    

			// exit if user cancels
			if (fd.getFile() == null) {
				System.out.println("presdec: No prc file selected");
				System.exit(0);
			}
			options.put("prcfile", new File(fd.getDirectory()+File.separator+fd.getFile()));
		}

		//
		// Check the supplied Filename
		//
		if(!((File)options.get("prcfile")).isFile()) {
			// maybe the extension is missing
			File tmpfile = new File(options.get("prcfile").toString()+".prc");
			if(!tmpfile.isFile()) {
				fail(options.get("prcfile").toString()+" is not a legal file");
			}
			options.put("prcfile", tmpfile);
		}

		//
		// If no path is specified, take the current directory
		//
		if(!options.containsKey("path")) {
			options.put("path", new File("."));  // This might be to system specific!
		}

		//
		// build output name if not supplied
		//
		if(!options.containsKey("output")) {
			String fn, bn;
			int c;
			fn = ((File)options.get("prcfile")).getName();
			c = fn.lastIndexOf('.');
			if (c>0) {
				bn = fn.substring(0,c);
			}
			else { // filename has no extension
				bn = fn;
			}
			options.put("output", new File(options.get("path") + File.separator + bn + ".rcp"));
		}

		// 
		// Default Bitmap converter is PbmAscii
		//
		if(!options.containsKey("bitmapconv")) {
			options.put("bitmapconv", new PbmAsciiConverter());
		}

		if(verbose) {
			System.out.println("\npresdec called with the following options:");
			System.out.println(options.toString());
			System.out.println("\n");
		}

		///////////////////////////////////
		// Read and process the prc file //
		///////////////////////////////////

		// Read the file and split it into seperate resource objects
		if(verbose) System.out.println("Loading " + options.get("prcfile"));
		try {
			prc = new PrcFile(options);
		}
		catch(Exception exc) {
			fail("IOError while reading " + options.get("prcfile"), exc);
		}

		// save all resources as binary files
		if(options.containsKey("binary")) {
			if(verbose) System.out.println("Saving binary resources");
			try {
				prc.saveRes();
			}
			catch (IOException ioexc) {
				fail("presdec: Error writing binary resource", ioexc);
			}
		}
		
		// Dump all resources that support dumping
		if(options.containsKey("dumpres")) {
			if(verbose) System.out.println("Dumping resources");
			try {
				prc.dumpRes();
			}
			catch (IOException ioexc) {
				fail("presdec: Error dumping resources");
			}
		}
		
		// Print structure of PRC file
		if(options.containsKey("structure")) {
			if(verbose) System.out.println("Printing structure info");
			System.out.println(prc.toString());
		}

		// build the rcp file 
		if(!(options.containsKey("NOPRC"))) {
			if(verbose) System.out.println("Building " + options.get("output"));
			FileWriter rcpwriter = null;
			try {
				rcpwriter = new FileWriter((File)options.get("output"));
			}
			catch (IOException ioexc) {
				fail("Error opening output file "+options.get("output"), ioexc);
			}

			// Sort the Array of Resources
			if(options.containsKey("sort")) {
				prc.sort();
			}
			// Write everything to the rcp file
			try {
				prc.writeRCP(rcpwriter);
				rcpwriter.close();
			}
			catch(Exception exc) {
				fail("Error building rcp file "+options.get("output"),exc);
			}
		}
		// exit the application. This is necessary, or the application will 
		// lock up (CTRL-C required) if the GUI file selector has been used. 
		System.exit(0);
    }

// End of main method ----------------------------------------------------

// Public methods --------------------------------------------------------	

	/**
	 *	Prints the help text
	 */
	public static void usage() {
		System.out.println("presdec: Convert a PalmPilot prc file into a rcp file suitable for pilrc");
		System.out.println("(c) 1998-1999 T.Holland");
		System.out.println("Version 1.2 22-jun-1999");
		System.out.println("");
		System.out.println("USAGE: java presdec [Options] Prcfile.prc");
		System.out.println("Options:");
		System.out.println("	-a      Generate all attributes, even if they are defaults");
//		System.out.println("	-A      Generate attributes that are unsupported by Pilrc");
		System.out.println("	-b      Save resources that are not part of the rcpfile as binary");
		System.out.println("	-B      Save all resources as binary");
		System.out.println("    -d      Dump resources in a human readable way (data resource only)");
//		System.out.println("*   -g      Start presdec with GUI");
		System.out.println("	-h      This message");
//		System.out.println("*	-i file Use this C style include file to get symbolic names for IDs");
//		System.out.println("*	-m file Build a gmake makefile suitable for rebuilding the PRC file");
//		System.out.println("	        This option implies the -b option");
		System.out.println("	-o file Name of the rcp output file");
		System.out.println("	        Default is the basename of the PRC file with a .rcp suffix");
		System.out.println("	-O      Do not generate rcp ouput file");
		System.out.println("	-p path	Path for the rcp file and all extracted resources");
		System.out.println("	-s      Sort the resources in the rcp file according to type and id");
		System.out.println("	-t type	Format for bitmaps and icons");
		System.out.println("	        Supported types: pbmascii, ascii, bmp, none");
		System.out.println("	-v      Print diagnostic messages to stdout");
		System.out.println("	-V      Print the structure of the PRC file to stdout");
//		System.out.println("* = not yet implemented");
	}

	/**
	 *	Print error message and exit 
	 *
	 *	@param	str	Error message
	 *	@param  exc	The Exception that caused the abort
	 *
	 *	@returns	The programm will exit!
	 */
	public static void fail(String str, Exception exc) {
		System.out.println("presdec: " + str);
		System.out.println("         reason: "+exc.getMessage());
		System.exit(-1);
	}

	/**
	 *	Print error message and exit 
	 *
	 *	@param	str	Error message
	 *
	 *	@return		The programm will exit!
	 */
	public static void fail(String str) {
		System.out.println("presdec: " + str);
		System.exit(-1);
	}

// End of public methods -------------------------------------------------

// Private methods -------------------------------------------------------

} // end of PrcFile class

/* Does not work yet
class PrcFilter implements FilenameFilter {

	PrcFilter() { // empty constructor
	}

	public boolean accept(File dir, String name) {
System.out.println("PrcFilter.accept("+dir+","+name+")");
		return name.endsWith(".prc");
	}
}
*/
