/*
 *  @(#) Res_MBAR.java 1.0 30-sep-98
 *
 *  Copyright by <ADRESS>Thomas Holland, innot@frankfurt.netsurf.de</ADRESS>
 *  This is part of the <I>presdec</I> application.
 *  For detailed information read the <TT>README</TT> file supplied with this application.
 *
 */

package presdec.resource;
 
import java.io.IOException;
import java.io.Writer;
import java.util.Hashtable;

import presdec.PrcFile;
import presdec.resource.ResByte;
import presdec.resource.menu.*;

/**
 * 	MBAR Resource
 *	<p>
 *	Containes a PalmOS menu resource and all of its descendantes (MenuItems)
 *
 *	@author Thomas Holland
 *	@version 1.0  30-sep-98
 *
 *	@see	presdec.resource.Resource
 *	@see	presdec.resource.menu.MenuPulldown
 */
public class Res_MBAR extends Resource {

// Data members ----------------------------------------------------------

	/** Flag if the menu is visible, not supported in pilrc */
	protected boolean visible;

	/** The current menu item, not supported in pilrc (always -1) */
	protected int curItem;

	/** Number of pulldown menus in the menubar*/
	protected int numMenus;

	/** The pulldown menus */
	protected MenuPulldown menuPulldown[];

	/** Size of struct MENUBAR */
	final static int SIZEMENUBAR = 32;

	/** Size of struct MENUPULLDOWN */
	final static int SIZEPULLDOWN = 34;
 
// End of data members ---------------------------------------------------
// Constructors ----------------------------------------------------------

	/** 
	 *	Construct a MBAR Resource object out of a byte array
	 *
	 *	Since some data is stored in the ResourceMapEntry and not in the 
	 *	Resource itself they have to be passed as arguments to the constructor
	 *
	 *	@param type	Type of resource as 4 letter String
	 *	@param id	ID of the resource
	 *	@param prc	Bytearray of the PRC file
	 *	@param offset   Starting point of the resource in the prc file
	 *	@param size	Size of the resource
	 *
	 *	@return		void
	 *  
	 */
	public Res_MBAR(int id, byte[] prc, int offset, int size) {
		super("MBAR", id, prc, offset, size);
		this.rcpitem = true;
		this.order   = MENU;

		try {
			// Skip over the first 4 longs
			resource.setPtr(16);

			// Get the attribute byte, currently no attributes are supported by pilrc
			int tmp = resource.getNextByte();
			if((tmp & 0x80)==0) 
				visible = true;
			else 
				visible = false;
			
			// skip pad byte and the curMenu word
			resource.getNextByte();
			resource.getNextWord();

			// Get the current Item, not supported by pilrc and always -1 anyway
			curItem = resource.getNextWord();

			// another unused long
			resource.getNextLong();

			// and the number of pulldown menus
			numMenus = resource.getNextWord();

			// one final unused long to bring us to the end of the struct
			resource.getNextLong();

			// build the array of pulldownMenus
			menuPulldown = new MenuPulldown[numMenus];
			for(int i=0; i<numMenus; i++) {
				menuPulldown[i] = new MenuPulldown(resource, SIZEMENUBAR + i*SIZEPULLDOWN);
			}
				
		}
		catch(ArrayIndexOutOfBoundsException exc) {
			resOK = false;
		}
	}			


// End of Constructors ---------------------------------------------------
// Public methods --------------------------------------------------------


	/**
	 *	Save Resource as RCP elements
	 * 
	 * 
	 *	@param writer	where the ascii description should be saved
	 *	
	 *  	@return 	boolean if the resource is not saveable in a RCP
	 *					
	 * 	@exception	java.io.IOException If the resource could not be written
	 *
	 */
	public boolean saveRCP(Writer writer) throws java.io.IOException {

		writer.write("MENU ID " + id + "\n");

		if(!resOK) {
			writer.write("presdec: Malformed Menubar resource\n");
			return true;
		}
		
		writer.write("BEGIN\n");

		for(int i=0; i<numMenus; i++) {
			menuPulldown[i].saveRCP(writer);
		}
		writer.write("END\n\n");

		return true;
	}


	/**
	 *	Generate a String with Type, ID and Size of the resource for debugging
	 */
//	public String toString() {
//		return "MBAR: numMenus " + numMenus;
//	}

// End of public methods -------------------------------------------------


} // end of class MBAR

