/*
 *  @(#) Res_Talt.java 1.0 30-sep-98
 *
 *  Copyright by <ADRESS>Thomas Holland, innot@frankfurt.netsurf.de</ADRESS>
 *  This is part of the <I>presdec</I> application.
 *  For detailed information read the <TT>README</TT> file supplied with this application.
 *
 */

package presdec.resource;
 
import java.io.IOException;
import java.io.Writer;
import java.util.Hashtable;

import presdec.PrcFile;
import presdec.resource.ResByte;

/**
 * 	Talt Resource
 *	<p>
 *	Containes a PalmOS alert
 *
 *	@author Thomas Holland
 *	@version 1.0  30-sep-98
 *
 *	@see	presdec.resource.Resource
 */
public class Res_Talt extends Resource {

// Data members ----------------------------------------------------------

	/** Type of Alert, 0=Information,1=Confirmation,2=Warning,3=Error */
	private int alertType;

	/** ID of the associated help resource */
	private int helpRscID;

	/** Number of buttons */
	private int numButtons;

	/** Default button */
	private int defButton;

	/** Title of the alert */
	private String title;
	
	/** The message body */
	private String message;

	/** Text of the buttons */
	private String buttonText[];
 
// End of data members ---------------------------------------------------
// Constructors ----------------------------------------------------------

	/** 
	 *	Construct a tALT Resource object out of a byte array
	 *
	 *	Since some data is stored in the ResourceMapEntry and not in the 
	 *	Resource itself they have to be passed as arguments to the constructor
	 *
	 *	@param type	Type of resource as 4 letter String
	 *	@param id	ID of the resource
	 *	@param prc	Bytearray of the PRC file
	 *	@param offset   Starting point of the resource in the prc file
	 *	@param size	Size of the resource
	 *
	 *  	@return		void
	 *
	 */
	public Res_Talt(int id, byte[] prc, int offset, int size) {
		super("Talt", id, prc, offset, size);
		this.rcpitem = true;
		this.order   = ALERT;

		try {
			// extract Type, HelpID, number of buttons and the default button
			alertType  = resource.getNextWord();
			helpRscID  = resource.getNextWord();
			numButtons = resource.getNextWord();
			defButton  = resource.getNextWord();

			// The remainder of the resource containes C type strings

			// extract title and body
			title = resource.getNextString(true);
			message = resource.getNextString(true);
		
			// finally the buttons
			buttonText = new String[numButtons];
			for(int i=0; i<numButtons; i++) {
				buttonText[i] = resource.getNextString(true);
			}
		}
		catch(ArrayIndexOutOfBoundsException exc) {
			resOK = false;
		}
	}			


// End of Constructors ---------------------------------------------------
// Public methods --------------------------------------------------------


	/**
	 *	Save Resource as RCP elements
	 * 
	 *
	 *	@param writer	where the ascii description should be saved
	 *	
	 *  	@return 	boolean if the resource is not saveable in a RCP
	 *					
	 * 	@exception	java.io.IOException If the resource could not be written
	 *
	 */
	public boolean saveRCP(Writer writer) throws java.io.IOException {

		writer.write("ALERT ID " + id + "\n");

		if(!resOK) {
			writer.write("presdec: Malformed Alert resource\n");
			return true;
		}

		if(helpRscID != 0)
			writer.write("HELPID " + helpRscID + "\n");

		switch(alertType) {
			case 0:	writer.write("INFORMATION\n");
					break;
			case 1:	writer.write("CONFIRMATION\n");
					break;
			case 2:	writer.write("WARNING\n");
					break;
			case 3:	writer.write("ERROR\n");
					break;
			default: writer.write(">presdec: Unknown Alert Type<\n");
		}

		writer.write("BEGIN\n");
		if(title.length()>0)
			writer.write("    TITLE       " + ResByte.splitString(title, 60, 16));
		
		if(message.length()>0)
			writer.write("    MESSAGE     " + ResByte.splitString(message, 60, 16));
		else // is it legal to have no MESSAGE TOKEN ?
			writer.write("    MESSAGE     " + "\"[Empty Message Text generated by presdec]\"\n");
			
		writer.write("    BUTTONS     "); 
		for(int i=0; i<numButtons; i++) 
			writer.write("\"" + buttonText[i] + "\" ");
		writer.write("\n");

		writer.write("END\n\n");

		return true;
	}


	/**
	 *	Generate a String with Type, ID and Size of the resource for debugging
	 */
//	public String toString() {
//		return "Talt: alertType " + alertType + "  numButtons " + numButtons;
//	}

// End of public methods -------------------------------------------------


} // end of class Talt

