/*
 *  @(#) Res_Tbmp.java	1.1 19-jun-99
 *
 *  Copyright by <ADRESS>Thomas Holland, innot@frankfurt.netsurf.de</ADRESS>
 *  This is part of the <I>presdec</I> application.
 *  For detailed information read the <TT>README</TT> file supplied with this application.
 *
 */

package presdec.resource;
 
import java.io.IOException;
import java.io.Writer;
import java.io.OutputStream;
import java.io.FileOutputStream;
import java.io.File;
import java.util.Hashtable;

import presdec.PrcFile;
import presdec.resource.ResByte;
import presdec.resource.bitmap.Converter;
import presdec.resource.bitmap.PilotBitmap;

/**
 * 	Tbmp Resource
 *	<p>
 *	Containes a bitmap
 *
 *	@author Thomas Holland
 *	@version 1.1  19-jun-99		Changed Writer to OutputStream for saving bitmaps
 *  	@version 1.0  30-sep-98		initial Version
 *
 *	@see	presdec.resource.Resource
 *	@see 	presdec.resource.bitmap.PilotBitmap
 *	@see	presdec.resource.bitmap.Converter
 */
public class Res_Tbmp extends Resource {

// Data members ----------------------------------------------------------

	/** The Bitmap */
	PilotBitmap bitmap;

	/** The name of the saved Bitmap file */
	public String name;

// End of data members ---------------------------------------------------
// Constructors ----------------------------------------------------------

	/** 
	 *	Construct a Tbmp Resource object out of a byte array
	 *
	 *	Since some data is stored in the ResourceMapEntry and not in the 
	 *	Resource itself they have to be passed as arguments to the constructor
	 *
	 *	@param type	Type of resource as 4 letter String
	 *	@param id	ID of the resource
	 *	@param prc	Bytearray of the PRC file
	 *  	@param offset   Starting point of the resource in the prc file
	 *  	@param size	Size of the resource
	 *
	 *	@return		void
	 *  
	 */
	public Res_Tbmp(int id, byte[] prc, int offset, int size) {
        	super("Tbmp", id, prc, offset, size);
		this.rcpitem = true;
		this.order   = BITMAP;

		bitmap = new PilotBitmap(resource);

	}


// End of Constructors ---------------------------------------------------
// Public methods --------------------------------------------------------


	/**
	 *	Save Resource as RCP elements
	 * 
	 * 
	 *	@param writer	where the ascii description should be saved
	 *	
	 *	@return		boolean if the resource is not saveable in a RCP
	 *					
	 * 	@exception	java.io.IOException If the resource could not be written
	 *
	 */
	public boolean saveRCP(Writer writer) throws IOException {
		
		// Get the needed options from PrcFile (as passed from Main)
		Hashtable options = PrcFile.sharedInstance().getOptions();
		File      path    = (File)options.get("path");
		Object    conv    = options.get("bitmapconv");	

		if(conv instanceof Converter) {	// A Converter was specified


			// Write the actual bitmap to a seperate file in the same directory...
			name = "bmp" + id + "." + ((Converter)conv).getExt();

			OutputStream outs = new FileOutputStream(path + File.separator + name);

			bitmap.saveBitmap(outs, (Converter)conv);

			outs.close();
		}
		else {	// No converter
			name = "[" + genResFilename() + "]"; // construct something
		}
		// ...and then a reference to this file in the rcp file
		writer.write("BITMAP ID " + id + " \"" + name + "\" ");
		if(bitmap.compressed) 
			writer.write("COMPRESS\n\n");
		else 
			writer.write("\n\n");

		return true;
	}


// End of public methods -------------------------------------------------


} // end of class Tbmp


