/*
 *  @(#) Res_data.java 1.1 25-jun-99
 *
 *  Copyright by <ADRESS>Markus Gutschke, markus@infoscape.com</ADRESS>
 *  This is part of the <I>presdec</I> application.
 *  For detailed information read the <TT>README</TT> file supplied with this application.
 *
 */

package presdec.resource;
 
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.io.File;
import java.util.Vector;
import java.util.Hashtable;

import presdec.PrcFile;
import presdec.resource.ResByte;

/**
 * 	data Resource
 *	<p>
 *	Contains the data segment
 *
 *	@author Markus Gutschke
 *	@version 1.1  25-jun-99 Modified to make data dumping optional (T.Holland)
 *	@version 1.0  11-apr-99
 *
 *	@see	presdec.resource.Resource
 */
public class Res_data extends Resource {

// Data members ----------------------------------------------------------

    // Error message
    public String error = null;

    // Offsets for data segment(s)
    public long dataOffset[];

    // Data segment(s)
    public byte data[][];

    // Relocation table for data segment
    public byte dataXRef[];

    // Relocation table for code segment
    public byte codeXRef[];

// End of data members ---------------------------------------------------
// Constructors ----------------------------------------------------------

	/** 
	 *	Construct a data Resource object out of a byte array
	 *
	 *	Since some data is stored in the ResourceMapEntry and not in the 
	 *	Resource itself they have to be passed as arguments to the constructor
	 *
	 *	@param type	Type of resource as 4 letter String
	 *	@param id	ID of the resource
	 *	@param prc	Bytearray of the PRC file
	 *	@param offset   Starting point of the resource in the prc file
	 *	@param size	Size of the resource
	 *
	 *	@return		void
	 *  
	 */
	public Res_data(int id, byte[] prc, int offset, int size) {
		super("data", id, prc, offset, size);
		this.rcpitem 	= false;
		this.dumpitem 	= true;
		this.order   	= DATA;
		try {
		    dataOffset     = new long[3];
		    data           = new byte[3][];
		    int codeOffset = resource.getNextLong();
		    for (int segment = 0; segment < 3; segment++) {
			dataOffset[segment] = resource.getNextLong();
			Vector bytes = new Vector();
			for (;;) {
			    int rle = resource.getNextByte();
			    if (rle >= 0x80)
				for (rle -= 0x80; rle-- >= 0;) bytes.add(new Byte((byte)resource.getNextByte()));
			    else if (rle >= 0x40)
				for (rle -= 0x40; rle-- >= 0;) bytes.add(new Byte((byte)0));
			    else if (rle >= 0x20) {
				Byte pattern = new Byte((byte)resource.getNextByte());
				for (rle -= 0x20; rle-- >= -1;) bytes.add(pattern); }
			    else if (rle >= 0x10)
				for (rle -= 0x10; rle-- >= -1;) bytes.add(new Byte((byte)0xFF));
			    else if (rle == 0x04) {
				bytes.add(new Byte((byte)0xA9));
				bytes.add(new Byte((byte)0xF0));
				bytes.add(new Byte((byte)0x00));
				bytes.add(new Byte((byte)resource.getNextByte()));
				bytes.add(new Byte((byte)resource.getNextByte()));
				bytes.add(new Byte((byte)resource.getNextByte()));
				bytes.add(new Byte((byte)0x00));
				bytes.add(new Byte((byte)resource.getNextByte())); }
			    else if (rle == 0x03) {
				bytes.add(new Byte((byte)0xA9));
				bytes.add(new Byte((byte)0xF0));
				bytes.add(new Byte((byte)0x00));
				bytes.add(new Byte((byte)0x00));
				bytes.add(new Byte((byte)resource.getNextByte()));
				bytes.add(new Byte((byte)resource.getNextByte()));
				bytes.add(new Byte((byte)0x00));
				bytes.add(new Byte((byte)resource.getNextByte())); }
			    else if (rle == 0x02) {
				bytes.add(new Byte((byte)0x00));
				bytes.add(new Byte((byte)0x00));
				bytes.add(new Byte((byte)0x00));
				bytes.add(new Byte((byte)0x00));
				bytes.add(new Byte((byte)0xFF));
				bytes.add(new Byte((byte)resource.getNextByte()));
				bytes.add(new Byte((byte)resource.getNextByte()));
				bytes.add(new Byte((byte)resource.getNextByte())); }
			    else if (rle == 0x01) {
				bytes.add(new Byte((byte)0x00));
				bytes.add(new Byte((byte)0x00));
				bytes.add(new Byte((byte)0x00));
				bytes.add(new Byte((byte)0x00));
				bytes.add(new Byte((byte)0xFF));
				bytes.add(new Byte((byte)0xFF));
				bytes.add(new Byte((byte)resource.getNextByte()));
				bytes.add(new Byte((byte)resource.getNextByte())); }
			    else if (rle == 0x00) {
            			data[segment] = new byte[bytes.size()];
            			for (int i = 0; i < data[segment].length; i++)
            			    data[segment][i] = ((Byte)bytes.elementAt(i)).byteValue();
            			bytes = null;
				break; }
			    else {
				throw new ArrayIndexOutOfBoundsException(); } } }
		    int dataXRefSize = codeOffset - resource.getPtr();
		    if (dataXRefSize < 6)
			throw new ArrayIndexOutOfBoundsException();
		    dataXRef = new byte[dataXRefSize];
		    for (int i = 0; i < dataXRefSize; i++)
			dataXRef[i] = (byte)resource.getNextByte();
		    int codeXRefSize = resource.size - resource.getPtr();
		    if (codeXRefSize < 6) {
			throw new ArrayIndexOutOfBoundsException(); }
		    codeXRef = new byte[codeXRefSize];
		    for (int i = 0; i < codeXRefSize; i++)
			codeXRef[i] = (byte)resource.getNextByte(); }
		catch (ArrayIndexOutOfBoundsException exc) {
			error = "[presdec: Malformed data segment in PRC file at offset "+ offset +"]";
			resOK = false;
		}
    }


// End of Constructors ---------------------------------------------------
// Public methods --------------------------------------------------------


	/**
	 *	Save Resource as binary
	 *
	 *  	@param filename	optional filename
	 *
	 *	@return		boolean success
	 * 
	 *	@exception	IOException If the resource could not be written 
	 *
	 */
	public boolean saveRes(String filename) throws IOException {

		FileOutputStream resout = new FileOutputStream(filename);
		resout.write(resource.toByteArray());
		resout.close();
		return true;
	}


	/**
	 *	Dump the resource as hexdump with additional reloc information.
	 *
	 *  	@param filename	optional filename
	 *
	 *	@return		boolean success
	 * 
	 *	@exception	IOException If the resource could not be written 
	 *
	 */
	 public boolean dumpRes(String filename) throws IOException {

		FileOutputStream resout = new FileOutputStream(filename);
		PrintStream out = new PrintStream(resout);
		if (error != null) {
			out.println("// " + error);
			return(true); 
		}
		for (int segment = 0; segment < 3; segment++) {
			long address = dataOffset[segment];
			String asciiString = "";
			if (data[segment].length > 0)
				out.println("// Data segment #" + segment + "\n");
		    	int i;
		    	for (i = 0; i < data[segment].length; i++) {
				if (i % 16 == 0) {
				String addressString;
				if (address < 0) {
					out.print("-");
					addressString = Long.toHexString(-address); 
				}
				else {
					out.print(" ");
					addressString = Long.toHexString( address); 
				}
			    	for (int j = addressString.length(); j < 8; j++)
					out.print("0");
			    		out.print(addressString + ":"); 
			    	}
				address++;
				if (i % 4 == 0)
			    		out.print(" ");
				int dataByte = ((int)data[segment][i])&0xFF;
				if (dataByte < 0x10)
					out.print(" 0" + Integer.toHexString(dataByte));
				else
					out.print(" "  + Integer.toHexString(dataByte));
				if (dataByte >= 0x20 && dataByte < 0x80)
					asciiString += new Character((char)dataByte);
				else
					asciiString += '.';
				if (i % 16 == 15) {
					out.println("  " + asciiString);
					asciiString = ""; 
				} 
			}
			if (asciiString.length() > 0) {
				for (; i % 16 != 0; i++) {
					if ((i-1) % 4 == 0)
						out.print(" ");
					out.print("   "); 
				}
				out.println("  " + asciiString); 
			}
			if (data[segment].length > 0)
				out.println(); 
		}
		out.println("// Data segment relocation table\n");
		int i;
		for (i = 0; i < dataXRef.length; i++) {
			int dataByte = ((int)dataXRef[i])&0xFF;
			if (dataByte < 0x10)
				out.print(" 0" + Integer.toHexString(dataByte));
			else
				out.print(" "  + Integer.toHexString(dataByte));
			if (i % 16 == 15)
				out.println(); 
		}
		if (i % 16 != 0)
			out.println();
		out.println("\n// Code segment relocation table\n");
		for (i = 0; i < codeXRef.length; i++) {
			int dataByte = ((int)codeXRef[i])&0xFF;
			if (dataByte < 0x10)
				out.print(" 0" + Integer.toHexString(dataByte));
			else
				out.print(" "  + Integer.toHexString(dataByte));
			if (i % 16 == 15)
				out.println(); 
		}
		if (i % 16 != 0)
			out.println();
		out.close();

		return true;
	} 
/**
	 *	Generate a String with Type, ID and Size of the resource for debugging
	 */
//	public String toString() {
//		return "tver: " + ver;
//	}

// End of public methods -------------------------------------------------


} // end of class Res_data
