/*
 *  @(#) Res_tFRM.java 1.0 30-sep-98
 *
 *  Copyright by <ADRESS>Thomas Holland, innot@frankfurt.netsurf.de</ADRESS>
 *  This is part of the <I>presdec</I> application.
 *  For detailed information read the <TT>README</TT> file supplied with this application.
 *
 */

package presdec.resource;
 
import java.io.IOException;
import java.io.Writer;
import java.util.Enumeration;
import java.util.Vector;
import java.util.Hashtable;

import presdec.PrcFile;
import presdec.resource.ResByte;
import presdec.resource.form.*;
import presdec.sort.Sorter;
import presdec.sort.FormComparer;

/**
 * 	tFRM Resource
 *	<p>
 *	Containes a PalmOS form resource and all of its form objects
 *
 *	@author Thomas Holland
 *  	@version 1.0  30-sep-98
 *
 *	@see	presdec.resource.Resource
 *	@see	presdec.resource.form.FormObject
 */
public class Res_tFRM extends Resource {

// Data members ----------------------------------------------------------

	/** Position and size of the form */
	protected int x, y, w, h;

	/** Flag if the Form has a frame */
	protected int frame;

	/** Diameter of Frame corners */
	protected int corndiam;

	/** Flag if the Form is modal */
	protected boolean modal;

	/** Form is a dialog, not supported by pilrc */
	protected boolean dialog;

	/** SaveBehind attribute */
	protected boolean savebehind;

	/** Usable attribute */
	protected boolean usable;

	/** The number of Objects in this Form */
	protected int numObjs;

	/** Array of all Objects in the Form */
	protected Vector formObjects;

	/** Help Resource ID */
	protected int helpID;

	/** Menu Resource ID */
	protected int menuID;

	/** Default Button */
	protected int defBtn;

	/** Size of Form header and FormObjectPointer */
	final static int SIZEFORMHEADER = 68;
	final static int SIZEFORMOBJPTR = 6;

	/** Types of form Objects */
	final static int RCFIELD 			= 0;
	final static int RCCONTROL 			= 1;
	final static int RCLIST 			= 2;
	final static int RCTABLE			= 3;
	final static int RCFORMBITMAP		= 4;
	//final static int RCLINE			= 5;
	//final static int RCFRAME			= 6;
	//final static int RCRECTANGLE		= 7;
	final static int RCLABEL			= 8;
	final static int RCTITLE			= 9;
	final static int RCPOPUP			= 10;
	final static int RCGRAFFITISTATE	= 11;
	final static int RCGADGET			= 12;
	final static int RCSCROLLBAR		= 13;



// End of data members ---------------------------------------------------
// Constructors ----------------------------------------------------------

	/** 
	 *	Construct a tFRM Resource object out of a byte array
	 *
	 *	Since some data is stored in the ResourceMapEntry and not in the 
	 *	Resource itself they have to be passed as arguments to the constructor
	 *
	 *	@param type	Type of resource as 4 letter String
	 *	@param id	ID of the resource
	 *	@param prc	Bytearray of the PRC file
	 *	@param offset   Starting point of the resource in the prc file
	 *	@param size	Size of the resource
	 *
	 *	@return		void
	 *  
	 */
	public Res_tFRM(int id, byte[] prc, int offset, int size) {
		super("tFRM", id, prc, offset, size);
		this.rcpitem = true;
		this.order   = FORM;

		try {
			int tmp;

			// The first 38 Bytes are the struct window, 
			// we need the size and the attributes

			// Skip over the first 4 Words
			resource.setPtr(8);

			// Get the attribute byte and extract what is supported by pilrc
			tmp = resource.getNextByte();
			if((tmp & 0x20)==0) 
				modal = false;
			else 
				modal = true;

			if((tmp & 0x02)==0) // dialog flag not supported by pilrc 
				dialog = false;
			else 
				dialog = true;
			
			// skip pad byte 
			resource.getNextByte();

			// get the dimensions of the Form
			x = resource.getNextWord();
			y = resource.getNextWord();
			w = resource.getNextWord();
			h = resource.getNextWord();

			// get the Frametype
			resource.setPtr(30);
			corndiam = resource.getNextByte(); // corner diameter, unused by pilrc
			tmp = resource.getNextByte();
			frame = (tmp & 0x03);	// 2 bits framewidth


			// Thats it for the struct window, continue with struct Form
			resource.setPtr(40);
			
			// ID is next, is this different from the resource ID?
			tmp = resource.getNextWord();
			if (tmp != id)
				System.out.println("Form ID differs from Resource ID");

			// Attributes of the From
			tmp = resource.getNextByte();
			if((tmp & 0x80)==0) 
				usable = false;
			else 
				usable = true;

			if((tmp & 0x08)==0) 
				savebehind = false;
			else 
				savebehind = true;

			// Continue with default button
			resource.setPtr(56);
			defBtn = resource.getNextWord();
			helpID = resource.getNextWord();
			menuID = resource.getNextWord();

			// finally the number of Objects in the form
			numObjs = resource.getNextWord();

		} // try
		catch(ArrayIndexOutOfBoundsException exc) {
			System.out.println("Exception occured: " + exc.toString());
			resOK = false;
		}
		
		// build the array of form objects
		int kind, start;
		formObjects = new Vector(numObjs);
		for(int i=0; i<numObjs; i++) {
			try {
				resource.setPtr(SIZEFORMHEADER + i * SIZEFORMOBJPTR);
				kind = resource.getNextByte(); // Type of Object
				resource.getNextByte();
				start = resource.getNextLong(); // and offset within the resource
				switch(kind) {
					case RCFIELD:	
								formObjects.addElement(new FormField(resource, start));
								break;
					case RCCONTROL:	
								formObjects.addElement(new FormControl(resource, start));
								break;
					case RCLIST:	
								formObjects.addElement(new FormList(resource, start));
								break;
					case RCTABLE:	
								formObjects.addElement(new FormTable(resource, start));
								break;
					case RCFORMBITMAP:	
								formObjects.addElement(new FormBitmap(resource, start));
								break;
					case RCLABEL:	
								formObjects.addElement(new FormLabel(resource, start));
								break;
					case RCTITLE:	
								formObjects.addElement(new FormTitle(resource, start));
								break;
					case RCPOPUP:	
								formObjects.addElement(new FormPopup(resource, start));
								break;
					case RCGRAFFITISTATE:
								formObjects.addElement(new FormGraffitistate(resource, start));
								break;
					case RCGADGET:	
								formObjects.addElement(new FormGadget(resource, start));
								break;
					case RCSCROLLBAR:
								formObjects.addElement(new FormScrollbar(resource, start));
								break;
					default:
								formObjects.addElement(new FormUnknown(resource, start, kind));
								break;
				} // select
			} // try
			catch(ArrayIndexOutOfBoundsException exc) {
				System.out.println("Exception occured: " + exc.toString());
				resOK = false;
			}
		} // for
	} // Constructor			


// End of Constructors ---------------------------------------------------
// Public methods --------------------------------------------------------



	/**
	 *	Save Resource as RCP elements
	 * 
	 * 
	 *	@param writer	where the ascii description should be saved
	 *	
	 *	@return 	boolean if the resource is not saveable in a RCP
	 *					
	 * 	@exception	java.io.IOException If the resource could not be written
	 *
	 */
	public boolean saveRCP(Writer writer) throws java.io.IOException {

		boolean all;

		Hashtable options = PrcFile.sharedInstance().getOptions();
		if(options.containsKey("all")) 
			all = true;
		else
			all = false;

		writer.write("FORM ID " + id + " AT (" + x +" " + y + " " + w + " " + h +")\n");

		if(!resOK) {
			writer.write("presdec: Malformed Form resource\n");
			return true;
		}

		// Write all possible flags
		if(frame!=0) {
			if(all) writer.write("FRAME\n");
		}
		else {
			writer.write("NOFRAME\n");
		}

		if(modal) {
			writer.write("MODAL\n");
		}

		if(savebehind) {
			if(all) writer.write("SAVEBEHIND\n");
		}
		else {
			writer.write("NOSAVEBEHIND\n");
		}

		if(usable) {
			if(all) writer.write("USABLE\n");
		}

		// and the referenced resources
		if(helpID != 0)
			writer.write("HELPID " + helpID + "\n");

		if(menuID != 0)
			writer.write("MENUID " + menuID + "\n");

		if(defBtn != 0)
			writer.write("DEFAULTBTNID " + defBtn + "\n");

		writer.write("BEGIN\n");

		Enumeration objs = formObjects.elements();
		FormObject fo;
		while(objs.hasMoreElements()) {
			fo = (FormObject)objs.nextElement();
			fo.saveRCP(writer); 
		}
		writer.write("END\n\n");

		return true;
	}

	/**
	 *	Sort Resource objects (only necessary for tFRM resources)
	 *
	 */
	public void sort() {
		// first sort by y and x
		try {
			Sorter.sort(formObjects, new FormComparer());
		}
		catch (Exception exc) {	// Should not be possible!
			System.out.println("presdec: Internal Error, please contact the author!");
			System.out.println(exc);
			System.exit(-1);
		}

		// then find and move Popup Objects to their associated lists
		int i,n;
		for (i=0; i<formObjects.size(); i++) {
			if(formObjects.elementAt(i) instanceof FormPopup) {
				FormPopup popup = (FormPopup)formObjects.elementAt(i);
				// find the associated list
				int listid = popup.listid;
				for(n=0; n<formObjects.size(); n++) {
					if(((FormObject)formObjects.elementAt(n)).id == listid) {
						break;
					}
				}
				if(n<formObjects.size()) { // found associated list
					if(i!=n-1) { // move popup only if it is not already at correct position
						formObjects.removeElementAt(i);
						if(i<n) 
							formObjects.insertElementAt(popup,n-1);
						else
							formObjects.insertElementAt(popup,n);

						i--; // since the popup at i has moved somewhere else
					}
				}
			}
		}
	}

	/** 
	 *	Get a list of resources that should follow this resource in a sorted RCP file
	 *
	 *	This is only used by the tFRM resource
	 *
	 *	@return		Vector of resources that should follow
	 *
	 */
	public Enumeration getRelated() {
		Vector v = new Vector();
		// First we take the menu and help IDs (reverse order)
		v.addElement(new Resource("MBAR", menuID));
		v.addElement(new Resource("tSTR", helpID));

		// and then we look for FormBitmap objects (reverse order so that they correctly sorted later)
		for(int i=formObjects.size()-1; i>=0; --i) {
			if(formObjects.elementAt(i) instanceof FormBitmap) {
				FormBitmap fbm = (FormBitmap)formObjects.elementAt(i);
				v.addElement(new Resource("Tbmp", fbm.bitmap));
			}
		}
		return v.elements();
	} 


	/**
	 *	Generate a String with Type, ID and Size of the resource for debugging
	 */
//	public String toString() {
//		return "tFRM: numObjs " + numObjs;
//	}

// End of public methods -------------------------------------------------


} // end of class tFRM

