/*
 *  @(#) Res_tSTR.java 1.0 30-sep-98
 *
 *  Copyright by <ADRESS>Thomas Holland, innot@frankfurt.netsurf.de</ADRESS>
 *  This is part of the <I>presdec</I> application.
 *  For detailed information read the <TT>README</TT> file supplied with this application.
 *
 */

package presdec.resource;
 
import java.io.IOException;
import java.io.Writer;
import java.util.Hashtable;

import presdec.PrcFile;
import presdec.resource.ResByte;

/**
 * 	tSTR Resource
 *	<p>
 *	Containes a single string
 *
 *	@author Thomas Holland
 *  	@version 1.0  30-sep-98
 *
 *	@see	presdec.resource.Resource
 */
public class Res_tSTR extends Resource {

// Data members ----------------------------------------------------------

	/** The string */
	public String str;

// End of data members ---------------------------------------------------
// Constructors ----------------------------------------------------------

	/** 
	 *	Construct a tSTR Resource object out of a byte array
	 *
	 *	Since some data is stored in the ResourceMapEntry and not in the 
	 *	Resource itself they have to be passed as arguments to the constructor
	 *
	 *	@param type	Type of resource as 4 letter String
	 *	@param id	ID of the resource
	 *	@param prc	Bytearray of the PRC file
	 *	@param offset   Starting point of the resource in the prc file
	 *	@param size	Size of the resource
	 *
	 *	@return		void
	 *  
	 */
	public Res_tSTR(int id, byte[] prc, int offset, int size) {
		super("tSTR", id, prc, offset, size);
		this.rcpitem = true;
		this.order   = STRING;

		try {
			StringBuffer buf = new StringBuffer(size);
			// A string resource may contain more than one string
			while(true) {
				buf.append(resource.getNextString(true));
				if(resource.getPtr() < size) { // more strings follow
					buf.append('\000');
					continue;
				}
				break;	// This was the last string
			}
			str = buf.toString();
		}
		catch (ArrayIndexOutOfBoundsException exc) {
			str = "presdec: Malformed string resource";
			resOK = false;
		}
	}


// End of Constructors ---------------------------------------------------
// Public methods --------------------------------------------------------


	/**
	 *	Save Resource as RCP elements
	 * 
	 *
	 *	@param writer	where the ascii description should be saved
	 *	
	 *	@return		boolean if the resource is not saveable in a RCP
	 *					
	 * 	@exception	java.io.IOException If the resource could not be written
	 *
	 */
	public boolean saveRCP(Writer writer) throws java.io.IOException {
		writer.write("STRING ID " + id);
		// get the string aligned nicely
		if(id < 10)			writer.write("     ");
		else if(id < 100)   writer.write("    ");
		else if(id < 1000)  writer.write("   ");
		else if(id < 10000)	writer.write("  ");
		else				writer.write(" ");
		writer.write(ResByte.splitString(str,60,16) + "\n");
		return true;
	}


	/**
	 *	Generate a String with Type, ID and Size of the resource for debugging
	 */
//	public String toString() {
//		return "tSTR: " + str;
//	}

// End of public methods -------------------------------------------------


} // end of class tSTR


