/*
 *  @(#) Res_tver.java 1.0 30-sep-98
 *
 *  Copyright by <ADRESS>Thomas Holland, innot@frankfurt.netsurf.de</ADRESS>
 *  This is part of the <I>presdec</I> application.
 *  For detailed information read the <TT>README</TT> file supplied with this application.
 *
 */

package presdec.resource;
 
import java.io.IOException;
import java.io.Writer;
import java.util.Hashtable;

import presdec.PrcFile;
import presdec.resource.ResByte;

/**
 * 	tver Resource
 *	<p>
 *	Containes a short version string
 *
 *	@author Thomas Holland
 *	@version 1.0  30-sep-98
 *
 *	@see	presdec.resource.Resource
 */
public class Res_tver extends Resource {

// Data members ----------------------------------------------------------

	/** The version string */
	public String ver;

// End of data members ---------------------------------------------------
// Constructors ----------------------------------------------------------

	/** 
	 *	Construct a tver Resource object out of a byte array
	 *
	 *	Since some data is stored in the ResourceMapEntry and not in the 
	 *	Resource itself they have to be passed as arguments to the constructor
	 *
	 *	@param type	Type of resource as 4 letter String
	 *	@param id	ID of the resource
	 *	@param prc	Bytearray of the PRC file
	 *	@param offset   Starting point of the resource in the prc file
	 *	@param size	Size of the resource
	 *
	 *	@return		void
	 *  
	 */
	public Res_tver(int id, byte[] prc, int offset, int size) {
		super("tver", id, prc, offset, size);
		this.rcpitem = true;
		this.order   = VERSION;
		try {
			ver = resource.getNextString(false);
		}
		catch (ArrayIndexOutOfBoundsException exc) {
			ver = "[presdec: Malformed version string in PRC file at offset "+ offset +"]";
			resOK = false;
		}
    }


// End of Constructors ---------------------------------------------------
// Public methods --------------------------------------------------------


	/**
	 *	Save Resource as RCP elements
	 * 
	 *
	 *	@param writer	where the ascii description should be saved
	 *	
	 *	@return		boolean if the resource is not saveable in a RCP
	 *					
	 * 	@exception	java.io.IOException If the resource could not be wriiten
	 *
	 */
	public boolean saveRCP(Writer writer) throws java.io.IOException {
		writer.write("VERSION ID " + id + " \"" + ver + "\"\n\n");
		return false;
	}


	/**
	 *	Generate a String with Type, ID and Size of the resource for debugging
	 */
//	public String toString() {
//		return "tver: " + ver;
//	}

// End of public methods -------------------------------------------------


} // end of class tver

