/*
 *  @(#) AsciiConverter.java  1.1  19-jun-99
 *
 *  Copyright by <ADRESS>Thomas Holland, innot@frankfurt.netsurf.de</ADRESS>
 *  This is part of the <I>presdec</I> application.
 *  For detailed information read the <TT>README</TT> file supplied with this application.
 *
 */

package presdec.resource.bitmap;

import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import presdec.resource.bitmap.Converter;

/**
 *	Write the PalmOS bitmap resource as a ASCII file for easy viewing with a text editor
 * 
 *	@author 	Thomas Holland
 *	@version	1.1 19-jun-99	changed to OutputStream iso Writer for saveing
 *	@version	1.0 30-sep-98
 *	@see 		presdec.resource.Res_Tbmp
 *	@see 		presdec.resource.bitmap.PilotBitmap
 *	@see		presdec.resource.bitmap.Converter
 */

public class AsciiConverter implements Converter {

	/**
	 *	Convert the supplied array of pixels into a visual ASCII representaionand write it
	 *	out to the supplied <code>OutputStream</code>.
	 *	
	 *	@param		pix	two-dimensional array of pixel values (for b/w bitmaps a pixel
	 *				may be <code>0</code> or <code>1</code>.
	 *	@param		x	number of columns in the pixel array.
	 *	@param		y	number of rows in the pixel array.
	 *	@param		depth	number of bits per pixel, currently set to 1 (b/w).
	 *	@param		outs	to write the generated data into.
	 *
	 *	@exception	IOException	as generated by the <code>OutputStream</code>
	 */
	public void save(int pix[][], int x, int y, int depth, OutputStream outs) throws IOException {

		// Since we are only writing ASCII, its probably easier to open a Writer
		OutputStreamWriter w = new OutputStreamWriter(outs);
		
		w.write("# CREATOR: presdec v1.0 (30-sep-98), extracted from a PalmPilot Resource file\n");
		w.write("Size " + x + "x" + y + "\n");

		w.write("+");
		for(int i=0; i<x; i++) w.write("-");
		w.write("+\n");

		for(int yc=0; yc<y; yc++) {
			w.write("|");
			for(int xc=0; xc<x; xc++) {
				if(pix[xc][yc]==0) w.write(" ");
				else w.write("*");
			}
			w.write("|\n");
		}

		w.write("+");
		for(int i=0; i<x; i++) w.write("-");
		w.write("+");
		
		w.close();

	}

	/**
	 *	return file extension for the ASCII.
	 *	
	 *	@return		String "asc"
	 *
	 */
	public String getExt() {
		return "asc";
	}
}

