/*
 *  @(#) Converter.java  1.1  19-jun-99
 *
 *  Copyright by <ADRESS>Thomas Holland, innot@frankfurt.netsurf.de</ADRESS>
 *  This is part of the <I>presdec</I> application.
 *  For detailed information read the <TT>README</TT> file supplied with this application.
 *
 */

package presdec.resource.bitmap;

import java.io.IOException;
import java.io.OutputStream; 


/**
 *	This interface is used to write Classes that convert PalmOS bitmap resources into
 *	different bitmap formats such as <code>PBM</code> or <code>BMP</code>.
 *	<p>
 *	Don't forget to add new Converters to presdec.Main
 *  
 *	@author 	Thomas Holland
 *	@version	1.1 19-jun-99	changed to OutputStream iso Writer for saveing
 *	@version	1.0 30-sep-98
 *	@see 		presdec.resource.Res_Tbmp
 *	@see 		presdec.resource.bitmap.PilotBitmap
 *	@see		presdec.resource.bitmap.PbmAsciiConverter
 *
 *	@see		presdec.Main#main
 */

public interface Converter {
	/**
	 *	Convert the supplied array of pixels into another format and write it out to
	 *	the supplied <code>OutputStream</code>.
	 *	
	 *	@param		pix	two-dimensional array of pixel values (for b/w bitmaps a pixel
	 *				may be <code>0</code> or <code>1</code>.
	 *	@param		x	number of columns in the pixel array.
	 *	@param		y	number of rows in the pixel array.
	 *	@param		depth	number of bits per pixel, currently set to 1 (b/w).
	 *	@param		outs	to write the generated data into.
	 *
	 *	@exception	IOException	as generated by the <code>OutputStream</code>
	 */
	public void save(int pix[][], int x, int y, int depth, OutputStream outs) throws IOException;

	/**
	 *	return a file extension describing the generated bitmap file, e.g. "pbm" or "bmp"
	 *	
	 *	@return		a String containg the associated file extension
	 *
	 */
	public String getExt();
}
