/*
 *  @(#) PbmAsciiConverter.java  1.1  19-jun-99
 *
 *  Copyright by <ADRESS>Thomas Holland, innot@frankfurt.netsurf.de</ADRESS>
 *  This is part of the <I>presdec</I> application.
 *  For detailed information read the <TT>README</TT> file supplied with this application.
 *
 */

package presdec.resource.bitmap;

import java.io.IOException;
import java.io.Writer;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import presdec.resource.bitmap.Converter;

/**
 *	Write the PalmOS bitmap resource as a PBM file, Type 1 (ASCII)
 *  
 *	@author 	Thomas Holland
 *	@version	1.1 19-jun-99	changed to OutputStream iso Writer for saveing
 *	@version	1.0 30-sep-98
 *	@see 		presdec.resource.Res_Tbmp
 *	@see 		presdec.resource.bitmap.PilotBitmap
 *	@see		presdec.resource.bitmap.Converter
 */

public class PbmAsciiConverter implements Converter {

	/**
	 *	Convert the supplied array of pixels into PBM Type 1 format and write it out to
	 *	the supplied <code>OutputStream</code>.
	 *	
	 *	@param		pix	two-dimensional array of pixel values (for b/w bitmaps a pixel
	 *				may be <code>0</code> or <code>1</code>.
	 *	@param		x	number of columns in the pixel array.
	 *	@param		y	number of rows in the pixel array.
	 *	@param		depth	number of bits per pixel, currently set to 1 (b/w).
	 *	@param		outs	to write the generated data into.
	 *
	 *	@exception	IOException	as generated by the <code>OutputStream</code>
	 */
	public void save(int pix[][], int x, int y, int depth, OutputStream outs) throws IOException {

		// Since we are only writing ASCII, its probably easier to open a Writer
		OutputStreamWriter w = new OutputStreamWriter(outs);

		w.write("P1\n");
		w.write("# CREATOR: presdec v1.0 (30-sep-98), extracted from a PalmPilot Resource file\n");
		w.write(x + " " + y + "\n");
		
		for(int yc=0; yc<y; yc++) {
			for(int xc=0; xc<x; xc++) {
				w.write(pix[xc][yc] + " ");
			}
			w.write("\n");
		}
		w.close();
	}

	/**
	 *	return file extension for a PBM type 1 file.
	 *	
	 *	@return		String "pbm"
	 *
	 */
	public String getExt() {
		return "pbm";
	}
}
