/*
 *  @(#) FormBitmap.java 1.0 30-sep-98
 *
 *  Copyright by <ADRESS>Thomas Holland, innot@frankfurt.netsurf.de</ADRESS>
 *  This is part of the <I>presdec</I> application.
 *  For detailed information read the <TT>README</TT> file supplied with this application.
 *
 */

package presdec.resource.form;
 
import java.io.Writer;
import java.io.IOException;

import presdec.resource.ResByte;

/**
 *  	A Bitmap ID object within a Form Resource.
 *
 *	The actual bitmap is in a Bitmap resource with the referenced ID.
 *
 *	@author Thomas Holland
 *  	@version 1.0  30-sep-98
 *
 *	@see	presdec.resource.form.FormObject
 *	@see	presdec.resource.Res_tFRM
 *	@see	presdec.resource.Res_Tbmp
 */
public class FormBitmap extends FormObject{

// Data members ----------------------------------------------------------

	/** Additional flags for the bitmap object */
	protected boolean usable;

	/** The Bitmap ID */
	public int bitmap;

// End of data members ---------------------------------------------------

// Constructors ----------------------------------------------------------

	/** 
	 *	Construct a formBitmap object out of a form resource
	 *
	 *	@param res	The resource
	 *	@param offset	Starting point of the object
	 *
	 *  	@return		void
	 *  
	 *	@exception	ArrayIndexOutOfBoundsException If the resource is to short
	 *			(reading past the end)
	 */
	public FormBitmap(ResByte res, int offset) throws ArrayIndexOutOfBoundsException {

		super(); // get options

		int tmp;		

		res.setPtr(offset);
		
		// Analyse the field attributes
		tmp = res.getNextByte();
		if ((tmp & 0x80)==0) usable = false; 
		else usable = true;
		res.getNextByte(); //pad

		// Position
		x = res.getNextWord();
		y = res.getNextWord();

		// And the Bitmap ID
		bitmap = res.getNextWord();

	}	

// End of Constructor ----------------------------------------------------

// Public methods --------------------------------------------------------


	/**
	 *	Save FormBitmap as RCP file
	 *
	 * 
	 *	@param writer	where the ascii description should be saved
	 *	
	 *  	@return 	boolean if the resource is not saveable in a RCP
	 *					
	 * 	@exception	java.io.IOException If the object could not be written
	 *
	 */
	public boolean saveRCP(Writer writer) throws IOException {
			
		writer.write("        FORMBITMAP  AT ("+x+" "+y+") BITMAP " + bitmap + " ");

		if(usable) {
			if(all) writer.write("USABLE ");
		}
		else writer.write("NONUSABLE ");
		
		writer.write("\n");

		return true;
	}



// End of public methods -------------------------------------------------

} // end of class FormBitmap



