/*
 *  @(#) FormControl.java 1.0 30-sep-98
 *
 *  Copyright by <ADRESS>Thomas Holland, innot@frankfurt.netsurf.de</ADRESS>
 *  This is part of the <I>presdec</I> application.
 *  For detailed information read the <TT>README</TT> file supplied with this application.
 *
 */

package presdec.resource.form;
 
import java.io.Writer;
import java.io.IOException;

import presdec.resource.ResByte;

/**
 *  	A Control (Button/Checkbox/Trigger etc.) within a Form Resource
 *
 *	@author Thomas Holland
 *	@version 1.0  30-sep-98
 *
 *	@see	presdec.resource.form.FormObject
 *	@see	presdec.resource.Res_tFRM
 */
public class FormControl extends FormObject{

// Data members ----------------------------------------------------------

	/** Button label */
	protected String 	label;

	
	/** Additional flags for the field object */
	protected boolean 	disabled;
	protected boolean 	on;
	protected boolean 	leftanchor;
	protected int 		frame;

	/** Style of Button */
	protected int 		style;

	/** The font ID */
	protected int 		font;

	/** Group ID for some buttons */
	protected int 		group;


	/** Types of Buttons */
	final static int	BUTTON = 0;
	final static int	PUSHBUTTON = 1;
	final static int 	CHECKBOX = 2;
	final static int	POPUPTRIGGER = 3;
	final static int 	SELECTORTRIG = 4;
	final static int	REPEATBUTTON = 5;

	/** Types of Frames */
	final static int	F_NONE = 0;
	final static int	F_STANDARD = 1;
	final static int	F_BOLD = 2;
	final static int 	F_RECT = 3;


// End of data members ---------------------------------------------------

// Constructors ----------------------------------------------------------

	/** 
	 *	Construct a formControl object out of a form resource
	 *
	 *	@param res	The resource
	 *	@param offset	Starting point of the object
	 *
	 *	@return		void
	 *
	 *	@exception	ArrayIndexOutOfBoundsException If the resource is to short
	 *			(reading past the end)
	 */
	public FormControl(ResByte res, int offset) throws ArrayIndexOutOfBoundsException {

		super(); // get options

		int tmp;		

		res.setPtr(offset);

		// Id and dimensions
		id = res.getNextWord();
		x = res.getNextWord();
		y = res.getNextWord();
		w = res.getNextWord();
		h = res.getNextWord();

		// The button label
		res.getNextLong(); // This is only a runtime pointer, the real string follows the struct

		// Analyse the control attributes
		tmp = res.getNextByte();
		if ((tmp & 0x80)==0) usable = false; 
		else usable = true;

		if ((tmp & 0x40)==0) disabled = true;
		else disabled = false;

		if ((tmp & 0x10)==0) on = false;
		else on = true;

		if ((tmp & 0x08)==0) leftanchor = false;
		else leftanchor = true;
	
		frame = (tmp & 0x07);

		res.getNextByte(); // Pad

		// Style, font and group ID
		style = res.getNextByte();
		font = res.getNextByte();
		group = res.getNextByte();
		res.getNextByte();

		// The Button label follows the object data
		label = res.getNextString(true);

	}	

// End of Constructor ----------------------------------------------------

// Public methods --------------------------------------------------------


	/**
	 *	Save FormControl as RCP file
	 *
	 * 
	 *	@param writer	where the ascii description should be saved
	 *	
	 *  @return 		boolean if the resource is not saveable in a RCP
	 *					
	 * 	@exception	    java.io.IOException  If the object could not be written
	 *
	 */
	public boolean saveRCP(Writer writer) throws IOException {
			
		writer.write("        ");
		switch (style) {
			case BUTTON:		writer.write("BUTTON      ");
								break;
			case PUSHBUTTON:	writer.write("PUSHBUTTON  ");		
								break;
			case CHECKBOX:		writer.write("CHECKBOX    ");
								break;
			case POPUPTRIGGER:	writer.write("POPUPTRIGGER ");
								break;
			case SELECTORTRIG:	writer.write("SELECTORTRIGGER ");
								break;
			case REPEATBUTTON:	writer.write("REPEATBUTTON ");
								break;
			default:			writer.write("[presdec: illegal control type]");
								return false;
		}

		writer.write("\"" + label + "\" ID " + id + " AT ("+x+" "+y+" "+w+" "+h+") ");
//		writer.write("                     ");

		if(usable) {
			if(all) writer.write("USABLE ");
		}
		else writer.write("NONUSABLE ");
		
		if(disabled) writer.write("DISABLED ");

		if(leftanchor) {
			writer.write("LEFTANCHOR ");
		}
		else if(all) writer.write("RIGHTANCHOR ");

		if(style == BUTTON || style == REPEATBUTTON) {
			switch(frame) {
				case F_NONE:		writer.write("NOFRAME ");
									break;
				case F_STANDARD:	if(all) writer.write("FRAME ");
									break;
				case F_BOLD:		writer.write("BOLDFRAME ");
									break;
				default:			writer.write("[presdec: unsupported frame style] ");
			}
		}
			
		if(font != 0)
			writer.write("FONT " + font + " ");

		if(style == PUSHBUTTON || style == CHECKBOX)
			writer.write("GROUP " + group + " "); 

		if(style == CHECKBOX && on)
			writer.write("CHECKED ");

		writer.write("\n");

		return true;
	}



// End of public methods -------------------------------------------------

} // end of class FormControl



