/*
 *  @(#) FormField.java 1.0 30-sep-98
 *
 *  Copyright by <ADRESS>Thomas Holland, innot@frankfurt.netsurf.de</ADRESS>
 *  This is part of the <I>presdec</I> application.
 *  For detailed information read the <TT>README</TT> file supplied with this application.
 *
 */

package presdec.resource.form;

import java.io.Writer;
import java.io.IOException;
 
import presdec.resource.ResByte;

/**
 *  	A text field within a Form Resource
 *
 *	@author Thomas Holland
 *  	@version 1.0  30-sep-98
 *
 *	@see	presdec.resource.form.FormObject
 *	@see	presdec.resource.Res_tFRM
 */
public class FormField extends FormObject{

// Data members ----------------------------------------------------------
	
	/** Additional flags for the field object */
	protected boolean disabled;
	protected boolean editable;
	protected boolean singleline;
	protected boolean dynamicsize;
	protected boolean autoshift;
	protected boolean numeric;
	
	protected int underlined;
	protected int justification;

	/** Max. number of chars in field */
	protected int maxchars;

	/** The font ID */
	protected int font;

// End of data members ---------------------------------------------------

// Constructors ----------------------------------------------------------

	/** 
	 *	Construct a formField object out of a form resource
	 *
	 *	@param res	The resource
	 *	@param offset	Starting point of the object
	 *
	 *  	@return		void
	 *  
	 *	@exception	ArrayIndexOutOfBoundsException If the resource is to short
	 *			(reading past the end)
	 */
	public FormField(ResByte res, int offset) throws ArrayIndexOutOfBoundsException {

		super(); // get options

		int tmp;		

		res.setPtr(offset);
		
		// Id and dimensions
		id = res.getNextWord();
		x = res.getNextWord();
		y = res.getNextWord();
		w = res.getNextWord();
		h = res.getNextWord();

		// Analyse the field attributes
		tmp = res.getNextWord();
		if ((tmp & 0x8000)==0) usable = false; 
		else usable = true;

		if ((tmp & 0x2000)==0) editable = false;
		else editable = true;

		if ((tmp & 0x1000)==0) singleline = false;
		else singleline = true;

		if ((tmp & 0x0400)==0) dynamicsize = false;
		else dynamicsize = true;
	
		underlined = (tmp & 0x00c0)>>6;

		justification = (tmp & 0x0030)>>4;

		if ((tmp & 0x0008)==0) autoshift = false;
		else autoshift = true;

		//hasscrollbar = (tmp && 0x0004) ? true : false;

		if ((tmp & 0x0002)==0) numeric = false;
		else numeric = true;
		
		// Next the max chars 
		res.setPtr(offset + 28);
		maxchars = res.getNextWord();

		// And the font ID
		res.setPtr(offset + 38);
		font = res.getNextByte();

	}	

// End of Constructor ----------------------------------------------------

// Public methods --------------------------------------------------------


	/**
	 *	Save FormField as RCP file
	 *
	 * 
	 *	@param writer	where the ascii description should be saved
	 *	
	 *  	@return 	boolean if the resource is not saveable in a RCP
	 *
	 * 	@exception	java.io.IOException  If the object could not be written
	 *
	 */
	public boolean saveRCP(Writer writer) throws IOException {
			
		writer.write("        FIELD       ID " + id + " AT ("+x+" "+y+" "+w+" "+h+") ");

		if(usable) {
			if(all) writer.write("USABLE ");
		}
		else writer.write("NONUSABLE ");
		
		if(disabled) writer.write("DISABLED ");

		if(editable) {
			if(all) writer.write("EDITABLE ");
		}
		else writer.write("NONEDITABLE ");

		if(singleline) {
			if(all) writer.write("SINGLELINE ");
		}
		else writer.write("MULTIPLELINES ");

		switch(justification) {
			case 0:		if(all) writer.write("LEFTALIGN "); break;
			case 1:		writer.write("CENTERED "); break;
			case 2: 	writer.write("RIGHTALIGN "); break;
			default:	writer.write("[presdec: illegal justification] ");
		}
		
		if(underlined != 0)
			writer.write("UNDERLINED ");

		if(dynamicsize)
			writer.write("DYNAMICSIZE ");

		if(autoshift)
			writer.write("AUTOSHIFT ");
		
		if(numeric)
			writer.write("NUMERIC ");

		if(font != 0)
			writer.write("FONT " + font + " ");

		if(maxchars != 0)
			writer.write("MAXCHARS " + maxchars + " ");

		writer.write("\n");

		return true;
	}



// End of public methods -------------------------------------------------

} // end of class FormField



