/*
 *  @(#) FormLabel.java 1.0 30-sep-98
 *
 *  Copyright by <ADRESS>Thomas Holland, innot@frankfurt.netsurf.de</ADRESS>
 *  This is part of the <I>presdec</I> application.
 *  For detailed information read the <TT>README</TT> file supplied with this application.
 *
 */

package presdec.resource.form;

import java.io.Writer;
import java.io.IOException;
 
import presdec.resource.ResByte;

/**
 *	A label within a Form Resource
 *
 *	@author Thomas Holland
 *	@version 1.0  30-sep-98
 *
 *	@see	presdec.resource.form.FormObject
 *	@see	presdec.resource.Res_tFRM
 */
public class FormLabel extends FormObject{

// Data members ----------------------------------------------------------

	/** The font ID */
	protected int font;

	/** The Label text */
	protected String label;

// End of data members ---------------------------------------------------

// Constructors ----------------------------------------------------------

	/** 
	 *	Construct a formLabel object out of a form resource
	 *
	 *	@param res	The resource
	 *	@param offset	Starting point of the object
	 *
	 *  	@return		void
	 *  
	 *	@exception	ArrayIndexOutOfBoundsException If the resource is to short
	 *			(reading past the end)
	 */
	public FormLabel(ResByte res, int offset) throws ArrayIndexOutOfBoundsException {

		super(); // get options

		int tmp;		

		res.setPtr(offset);
		
		// Id and dimensions
		id = res.getNextWord();
		x = res.getNextWord();
		y = res.getNextWord();

		// Analyse the field attributes
		tmp = res.getNextByte();
		if ((tmp & 0x80)==0) usable = false; 
		else usable = true;
		res.getNextByte(); //pad

		// And the font ID
		font = res.getNextByte();
		res.getNextByte(); //pad

		// Get the string
		res.getNextLong(); // Pointer points to nowhere
		label = res.getNextString(true); // but string is right behind it

	}	

// End of Constructor ----------------------------------------------------

// Public methods --------------------------------------------------------


	/**
	 *	Save FormLabel as RCP file
	 *
	 * 
	 *	@param writer	where the ascii description should be saved
	 *	
	 *  	@return		boolean if the resource is not saveable in a RCP
	 *					
	 * 	@exception	java.io.IOException If the object could not be written
	 *
	 */
	public boolean saveRCP(Writer writer) throws IOException {
			
		writer.write("        LABEL       \"" + label + "\" ID " + id + " AT ("+x+" "+y+") ");

		if(usable) {
			if(all) writer.write("USABLE ");
		}
		else writer.write("NONUSABLE ");
		
		if(font != 0 || all)
			writer.write("FONT " + font + " ");

		writer.write("\n");

		return true;
	}



// End of public methods -------------------------------------------------

} // end of class FormLabel



