/*
 *  @(#) FormList.java 1.0 30-sep-98
 *
 *  Copyright by <ADRESS>Thomas Holland, innot@frankfurt.netsurf.de</ADRESS>
 *  This is part of the <I>presdec</I> application.
 *  For detailed information read the <TT>README</TT> file supplied with this application.
 *
 */

package presdec.resource.form;

import java.io.Writer;
import java.io.IOException;
 
import presdec.resource.ResByte;

/**
 *  	A list object within a Form Resource
 *
 *	@author Thomas Holland
 *  	@version 1.0  30-sep-98
 *
 *	@see	presdec.resource.form.FormObject
 *	@see	presdec.resource.Res_tFRM
 */

public class FormList extends FormObject{

// Data members ----------------------------------------------------------
	
	/** Additional flags for the list object */
	protected boolean disabled;
	
	protected int numitems;

	// unused as it can not be reconstructed
	// protected int numvisible;

	/** Array of all list items */
	protected String item[];

	/** The font ID */
	protected int font;

	/** Size of the list object (to find the items behind it) */
	final static int SIZELISTOBJECT = 32;

// End of data members ---------------------------------------------------

// Constructors ----------------------------------------------------------

	/** 
	 *	Construct a formField object out of a form resource
	 *
	 *	@param res	The resource
	 *	@param offset	Starting point of the object
	 *
	 *  	@return		void
	 *  
	 *	@exception	ArrayIndexOutOfBoundsException If the resource is to short
	 *			(reading past the end)
	 */
	public FormList(ResByte res, int offset) throws ArrayIndexOutOfBoundsException {

		super(); // get options

		int tmp;		

		res.setPtr(offset);
		
		// Id and dimensions
		id = res.getNextWord();
		x = res.getNextWord();
		y = res.getNextWord();
		w = res.getNextWord();
		h = res.getNextWord();

		// Analyse the field attributes
		tmp = res.getNextWord();
		if ((tmp & 0x8000)==0) usable = false; 
		else usable = true;

		if ((tmp & 0x4000)==0) disabled = true;
		else disabled = false;

		// items are found at the end of the object data
		res.getNextLong();

		// The number of items
		numitems = res.getNextWord();

		res.getNextLong(); // skip 2 words

		// And the font ID
		font = res.getNextByte();

		// Go to the list items
		// Between the object and the items there is one long for each item
		res.setPtr(offset + SIZELISTOBJECT + numitems*4);

		item = new String[numitems];
		for(int i=0; i<numitems; i++) {
			item[i] = res.getNextString(true);
		}
	}	

// End of Constructor ----------------------------------------------------

// Public methods --------------------------------------------------------


	/**
	 *	Save FormField as RCP file
	 *
	 * 
	 *	@param writer	where the ascii description should be saved
	 *	
	 *  	@return 	boolean if the resource is not saveable in a RCP
	 *					
	 * 	@exception	java.io.IOException If the object could not be written
	 *
	 */
	public boolean saveRCP(Writer writer) throws IOException {
			
		writer.write("        LIST        ");

		if(numitems == 0) { // list is empty
			writer.write("\"\" ");
		}

		for(int i=0; i<numitems; i++) {
			writer.write("\"" + item[i] + "\" ");
		}

		writer.write("ID " + id + " AT ("+x+" "+y+" "+w+" "+h+") ");

		if(usable) {
			if(all) writer.write("USABLE ");
		}
		else writer.write("NONUSABLE ");
		
		if(disabled) writer.write("DISABLED ");

		if(font != 0 || all)
			writer.write("FONT " + font + " ");

		writer.write("\n");

		return true;
	}



// End of public methods -------------------------------------------------

} // end of class FormList



