/*
 *  @(#) FormObject.java 1.0 30-sep-98
 *
 *  Copyright by <ADRESS>Thomas Holland, innot@frankfurt.netsurf.de</ADRESS>
 *  This is part of the <I>presdec</I> application.
 *  For detailed information read the <TT>README</TT> file supplied with this application.
 *
 */

package presdec.resource.form;
 
import java.io.Writer;
import java.io.IOException;
import java.util.Hashtable;

import presdec.PrcFile;
import presdec.resource.ResByte;

/**
 *	A object as part of a tFRM Resource.
 *	<p>
 *	This class containes some data members common to most form objects and
 *	is extended by the various actual form object types
 *
 *	@author Thomas Holland
 *  	@version 1.0  30-sep-98
 *
 *	@see	presdec.resource.Res_tFRM
 */
public abstract class FormObject {

// Public data members ---------------------------------------------------

	// These are the members common to most form objects
	/** ID of this Object */
	public int id;

	/** Object position and size*/
	public int x, y, w, h;
	
	/** Additional flags for the label object */
	public boolean usable;

	/** Flags for generation of RCP text, as set by options from Main */
	boolean verbose, all, unsupported;

// Public methods --------------------------------------------------------

	/**
	 *	The Constructor is called from each subclass and sets the options
	 *
	 */
	public FormObject() {
		Hashtable options = PrcFile.sharedInstance().getOptions();

		if(options.containsKey("all")) 
			all = true;
		else
			all = false;

		if(options.containsKey("ALL"))
			unsupported = true;
		else
			unsupported = false;

		if(options.containsKey("verbose"))
			verbose = true;
		else
			verbose = false;
	}
	

	/**
	 *	Save Object as RCP file
	 *
	 *  	Overwrite this in each subclass
	 * 
	 * 
	 *	@param writer	where the ascii description should be saved
	 *	@param all	true if default attributes should be written as well
	 *	
	 *  	@return 	boolean if the resource is not saveable in a RCP
	 *					
	 * 	@exception	java.io.IOException If the object could not be written
	 *
	 */
	abstract public boolean saveRCP(Writer writer) throws IOException; 

// End of public methods -------------------------------------------------

} // end of class FormObject



