/*
 *  @(#) FormScrollbar.java 1.0 30-sep-98
 *
 *  Copyright by <ADRESS>Thomas Holland, innot@frankfurt.netsurf.de</ADRESS>
 *  This is part of the <I>presdec</I> application.
 *  For detailed information read the <TT>README</TT> file supplied with this application.
 *
 */

package presdec.resource.form;

import java.io.Writer;
import java.io.IOException;
 
import presdec.resource.ResByte;

/**
 *	A scrollbar object within a Form Resource
 *
 *	@author Thomas Holland
 *	@version 1.0  30-sep-98
 *
 *	@see	presdec.resource.form.FormObject
 *	@see	presdec.resource.Res_tFRM
 */

public class FormScrollbar extends FormObject{

// Data members ----------------------------------------------------------

	/** Scrollbar values */
	protected int		value;
	protected int		min;
	protected int		max;
	protected int		pagesize;


// End of data members ---------------------------------------------------

// Constructors ----------------------------------------------------------

	/** 
	 *	Construct a formScrollbar object out of a form resource
	 *
	 *	@param res	The resource
	 *	@param offset	Starting point of the object
	 *
	 *	@return		void
	 *  
	 *	@exception	ArrayIndexOutOfBoundsException If the resource is to short
	 *			(reading past the end)
	 */
	public FormScrollbar(ResByte res, int offset) throws ArrayIndexOutOfBoundsException {

		super(); // get options

		int tmp;		

		res.setPtr(offset);
		
		// Get scrollbar dimensions and id
		x = res.getNextWord();
		y = res.getNextWord();
		w = res.getNextWord();
		h = res.getNextWord();
		id = res.getNextWord();

		// Analyse the field attributes
		tmp = res.getNextByte();
		if ((tmp & 0x80)==0) usable = false; 
		else usable = true;
		res.getNextByte(); //pad

		// get the scrollbar values
		value 	 = res.getNextWord();
		min   	 = res.getNextWord();
		max		 = res.getNextWord();
		pagesize = res.getNextWord();		

	}	

// End of Constructor ----------------------------------------------------

// Public methods --------------------------------------------------------


	/**
	 *	Save FormScrollbar as RCP file
	 *
	 * 
	 *	@param writer	where the ascii description should be saved
	 *	
	 *  	@return		boolean if the resource is not saveable in a RCP
	 *					
	 * 	@exception	java.io.IOException If the object could not be written
	 *
	 */
	public boolean saveRCP(Writer writer) throws IOException {
			
		writer.write("        SCROLLBAR   ID " + id + " AT ("+x+" "+y+" "+w+" "+h+") ");

		writer.write("VALUE " + value + " MIN " + min + " MAX " + max + " PAGESIZE " + pagesize + " ");

		if(usable) {
			if(all) writer.write("USABLE ");
		}
		else writer.write("NONUSABLE ");
		
		writer.write("\n");

		return true;
	}



// End of public methods -------------------------------------------------

} // end of class FormScrollbar



