/*
 *  @(#) FormTable.java 1.0 30-sep-98
 *
 *  Copyright by <ADRESS>Thomas Holland, innot@frankfurt.netsurf.de</ADRESS>
 *  This is part of the <I>presdec</I> application.
 *  For detailed information read the <TT>README</TT> file supplied with this application.
 *
 */

package presdec.resource.form;
 
import java.io.Writer;
import java.io.IOException;

import presdec.resource.ResByte;

/**
 *  	A Table within a Form Resource
 *
 *	@author Thomas Holland
 *  	@version 1.0  30-sep-98
 *
 *	@see	presdec.resource.form.FormObject
 *	@see	presdec.resource.Res_tFRM
 */
public class FormTable extends FormObject{

// Data members ----------------------------------------------------------

	/** Number of rows in table */
	protected int rows;

	/** Number of columns in table */
	protected int columns;
	
	/** width of the columns */
	protected int cwidth[];

	/** size of a column attribute field */
	final static int SIZECOLUMNATTR = 18;

	/** size of the Table header */
	final static int SIZETABLE = 74;

// End of data members ---------------------------------------------------

// Constructors ----------------------------------------------------------

	/** 
	 *	Construct a FormTable object out of a form resource
	 *
	 *	@param res	The resource
	 *	@param offset	Starting point of the object
	 *
	 *  	@return		void
	 *
	 *	@exception	ArrayIndexOutOfBoundsException If the resource is to short
	 *			(reading past the end)
	 *  
	 */
	public FormTable(ResByte res, int offset) throws ArrayIndexOutOfBoundsException {

		super(); // get options

		int tmp;		

		res.setPtr(offset);
		
		id = res.getNextWord();

		x = res.getNextWord();
		y = res.getNextWord();
		w = res.getNextWord();
		h = res.getNextWord();

		res.getNextByte();	// The row attributes
		res.getNextByte();	// padding

		columns = res.getNextWord();
		rows = res.getNextWord();

		res.getNextWord();	// current row
		res.getNextWord();	// current column
		res.getNextWord();	// top row
		
		res.getNextLong();	// column attrs

		res.getNextLong();	// row attrs
		res.getNextLong();	// item attrs
		res.getNextLong();	// current field

		// step thru all column attribute fields
		cwidth = new int[columns];
		for(int i=0; i<columns; i++) {
			res.setPtr(offset + SIZETABLE + i*SIZECOLUMNATTR);
			cwidth[i] = res.getNextWord();
			// other attributes not supported by pilrc
		}
	}	

// End of Constructor ----------------------------------------------------

// Public methods --------------------------------------------------------


	/**
	 *	Save FormTable as RCP file
	 *
	 * 
	 *	@param writer	where the ascii description should be saved
	 *	
	 *  	@return		boolean if the resource is not saveable in a RCP
	 *					
	 * 	@exception	java.io.IOException  If the object could not be written
	 *
	 */
	public boolean saveRCP(Writer writer) throws java.io.IOException {
			
		writer.write("        TABLE       ID " + id + " AT ("+x+" "+y+" "+w+" "+h+") ");

		writer.write("ROWS " + rows + " COLUMNS " + columns + " COLUMNWIDTHS ");
		
		for(int i=0; i<columns; i++) {
			writer.write(cwidth[i] + " ");
		}
		
		writer.write("\n");

		return true;
	}



// End of public methods -------------------------------------------------

} // end of class FormTable



