/*
 *  @(#) MenuPulldown.java 1.0 30-sep-98
 *
 *  Copyright by <ADRESS>Thomas Holland, innot@frankfurt.netsurf.de</ADRESS>
 *  This is part of the <I>presdec</I> application.
 *  For detailed information read the <TT>README</TT> file supplied with this application.
 *
 */

package presdec.resource.menu;
 
import java.io.*;
import presdec.resource.ResByte;

/**
 * 	MenuPulldown Resource
 *	<p>
 *	A PalmOS pulldown menu resource
 *
 *	@author Thomas Holland
 *	@version 1.0  30-sep-98
 *
 *	@see	presdec.resource.Res_MBAR
 */
public class MenuPulldown {
// Data members ----------------------------------------------------------

	/** Size of struct MENUPULLDOWN */
	final static int 	SIZEPULLDOWN = 34;

	/** Size of struct MENUITEM */
	final static int 	SIZEMENUITEM = 8;

	/** Title of the pulldown menu */
	protected String	Title;

	/** Number of menu items in the pulldown menu */
	protected int 		numMenuitems;

	/** Array of Menuitem IDs */
	protected int 		mi_ID[];

	/** Array of Menuitem shortcuts */
	protected byte 		mi_Command[];

	/** Array of Menuitem strings */
	protected String 	mi_itemStr[];

// End of data members ---------------------------------------------------
// Constructors ----------------------------------------------------------

	/** 
	 *	Construct a pulldown menu object out of a MBAR resource
	 *
	 *	@param mbar	The MBAR resource
	 *	@param offset	Starting point of the pulldown menu
	 *
	 *	@return		void
	 *  
	 *	@exception	ArrayIndexOutOfBoundsException If the resource is to short
	 *			(reading past the end)
	 */
	public MenuPulldown(ResByte mbar, int offset) throws ArrayIndexOutOfBoundsException {
	
		int p_mi;		// pointer to the menu items
		int p_str;		// pointer to a string
		int p_cur;		// for saving the current value of the ptr

		// Get the title of the pulldown menu
		mbar.setPtr(offset + 24);
		p_str = mbar.getNextLong();
		Title = mbar.getString(p_str, true);

		// and the number of menu items
		numMenuitems = mbar.getNextWord();

		// finally the pointer to the menuitems
		p_mi = mbar.getNextLong();

		// Extract all menuitems
		mi_ID      = new int[numMenuitems];
		mi_Command = new byte[numMenuitems];
		mi_itemStr = new String[numMenuitems];

		mbar.setPtr(p_mi);
		for(int i=0; i<numMenuitems; i++) {
			mi_ID[i]      = mbar.getNextWord();
			mi_Command[i] = (byte)mbar.getNextByte();
			mbar.getNextByte();		// pad
			p_str         = mbar.getNextLong();
			mi_itemStr[i] = mbar.getString(p_str, true);
		}
	}	

// End of Constructor ----------------------------------------------------

// Public methods --------------------------------------------------------


	/**
	 *	Save pulldown menu as RCP elements
	 * 
	 * 
	 *	@param writer	where the ascii description should be saved
	 *	
	 *  	@return		boolean if the resource is not saveable in a RCP
	 *					
	 * 	@exception	java.io.IOException If menu could not be written
	 *
	 */
	public boolean saveRCP(Writer writer) throws java.io.IOException {

		writer.write("        PULLDOWN \"" + Title + "\"\n");
		writer.write("        BEGIN\n");

		for(int i=0; i<numMenuitems; i++) {
			writer.write("                MENUITEM ");
			if(mi_itemStr[i].equals("-")) {	// Seperator Bar
				writer.write("SEPARATOR\n");
			}
			else
			{
				writer.write("\"" + mi_itemStr[i] + "\" ID " + mi_ID[i] + " ");
				if(mi_Command[i] != 0) 
					writer.write("\"" + (char)mi_Command[i] + "\"");
				writer.write("\n");
			}
		}
		writer.write("        END\n\n");

		return true;
	}

// End of public methods -------------------------------------------------

} // end of class MenuPulldown



