/*
 *  @(#) FormComparer.java 1.0 30-sep-98
 *
 *  Copyright by <ADRESS>Thomas Holland, innot@frankfurt.netsurf.de</ADRESS>
 *  This is part of the <I>presdec</I> application.
 *  For detailed information read the <TT>README</TT> file supplied with this application.
 *
 *  Based on <tt>StringComparer</tt> and the <tt>Sorter</tt> class from the <i>jd</i> application by Derek Lieber. 
 */

package presdec.sort;

import presdec.resource.form.FormObject;
import presdec.sort.Comparer;

public class FormComparer implements Comparer {
	/**
	 *	Compare two Form Objects by y and x coordinates..
	 *	(sort by rows and, if rows are equal, by columns).
	 *	This is needed for the Comparer interface.
	 *
	 *	@param 	aa		First Resource
	 *	@param	bb		Second Resource
	 *	@return			<code>true</code> if <i>aa</i> is lower than <i>bb</i>, 
	 *				otherwise <code>false</code>.
	 *	
	 *	@exception	Exception 	If one or both Objects are not Form Objects
	 *
	 */
  	public boolean isLessThan(Object aa, Object bb) throws Exception {
		if (aa instanceof FormObject && bb instanceof FormObject) {
			FormObject a = (FormObject)aa;
			FormObject b = (FormObject)bb;
			if(a.y < b.y) {
				return true;
			}
			else if(a.y == b.y) {
				if(a.x < b.x) {
					return true;
				}
			}

			return false;
		}
		throw new Exception("FormComparer: Object mismatch");
	}
      
	/**
	 *	Compare two Form Objects by y and x coordinates 
	 *	(sort by rows and, if rows are equal, by columns).
	 *	This is needed for the Comparer interface.
	 *
	 *	@param 	aa		First Resource
	 *	@param	bb		Second Resource
	 *	@return			<code>true</code> if <i>aa</i> is greater than <i>bb</i>, 
	 *					otherwise <code>false</code>.
	 *	
	 *	@exception	Exception 	If one or both Objects are not Resource Objects
	 *
	 */

   public boolean isGreaterThan(Object aa, Object bb) throws Exception {
		if (aa instanceof FormObject && bb instanceof FormObject) {
			FormObject a = (FormObject)aa;
			FormObject b = (FormObject)bb;
			if(a.y > b.y) {
				return true;
			}
			else if(a.y == b.y) {
				if(a.x > b.x) {
					return true;
				}
			}

			return false;
		}
		throw new Exception("FormComparer: Object mismatch");
	}
}
