/*
 *  @(#) ResourceComparer.java 1.0 30-sep-98
 *
 *  Copyright by <ADRESS>Thomas Holland, innot@frankfurt.netsurf.de</ADRESS>
 *  This is part of the <I>presdec</I> application.
 *  For detailed information read the <TT>README</TT> file supplied with this application.
 *
 *  Based on <tt>StringComparer</tt> and the <tt>Sorter</tt> class from the <i>jd</i> application by Derek Lieber. 
 */

package presdec.sort;

import presdec.resource.Resource;
import presdec.sort.Comparer;

public class ResourceComparer implements Comparer {
	/**
	 *	Compare two Resource Objects by order and ID.
	 *	This is needed for the Comparer interface.
	 *
	 *	@param 	aa		First Resource
	 *	@param	bb		Second Resource
	 *	@return			<code>true</code> if <i>aa</i> is lower than <i>bb</i>, 
	 *				otherwise <code>false</code>.
	 *	
	 *	@exception	Exception 	If one or both Objects are not Resource Objects
	 *
	 */
   	public boolean isLessThan(Object aa, Object bb) throws Exception {
		if (aa instanceof Resource && bb instanceof Resource) {
			Resource a = (Resource)aa;
			Resource b = (Resource)bb;
			if (a.order < b.order) {
				return true;
			}
			else if (a.order == b.order) {
				if (a.id < b.id) {
					return true;
				}
			}
			return false;
		}
		throw new Exception("ResourceComparer: Object mismatch");
	}
      
	/**
	 *	Compare two Resource Objects by order and ID.
	 *	This is needed for the Comparer interface.
	 *
	 *	@param 	aa		First Resource
	 *	@param	bb		Second Resource
	 *	@return			<code>true</code> if <i>aa</i> is higher than <i>bb</i>, 
	 *					otherwise <code>false</code>.
	 *	
	 *	@exception	Exception 	If one or both Objects are not Resource Objects
	 *
	 */
   public boolean isGreaterThan(Object aa, Object bb) throws Exception {
		if (aa instanceof Resource && bb instanceof Resource) {
			Resource a = (Resource)aa;
			Resource b = (Resource)bb;
			if (a.order > b.order) {
				return true;
			}
			else if (a.order == b.order) {
				if (a.id > b.id) {
					return true;
				}
			}
			return false;
		}
		throw new Exception("ResourceComparer: Object mismatch");
	}
}
