/*
 *  @(#) Sorter.java 1.0 30-sep-98
 *
 *  Copyright by <ADRESS>Thomas Holland, innot@frankfurt.netsurf.de</ADRESS>
 *  This is part of the <I>presdec</I> application.
 *  For detailed information read the <TT>README</TT> file supplied with this application.
 *
 *  Based on <tt>StringComparer</tt> and the <tt>Sorter</tt> class from the <i>jd</i> application by Derek Lieber. 
 */


package presdec.sort;

import java.util.Vector;
import presdec.sort.Comparer;

/**
 *    The static method @link #sort will sort any Vector of Objects 
 *    To compare the objects an Comparer Class is used.
 *    <p>
 *    The sorting algorithm used is a shell sort.
 *    <p>
 *
 *    @see Comparer
 *
 */
public class Sorter 
{
    /**
     * Sort an Vector of Objecs with the help of a Comparer function
     *
     * @param	x	Vector of Objects to be sorted
     * @param	cmp	Class implementing the Comparer Interface
     * @exception	Exception Thrown by the Comparer Class if the objects in
     *			the Vector can not be compared.
     *
     */
    public static void sort(Vector x, Comparer cmp) throws Exception
    {
	int n = x.size();
	for (int gap = n / 2; gap > 0; gap /= 2) {
	    for (int i = gap; i < n; ++i) {
		for (int j = i - gap; j >= 0; j -= gap)
		    {
			if (cmp.isLessThan(x.elementAt(j), x.elementAt(j + gap)))
			    break;
			swap(x, j, j + gap);
		    }
	    }
	}
    }
    
   private static void
       swap(Vector x, int i, int j)
    {
	Object tmp = x.elementAt(i);
	x.setElementAt(x.elementAt(j), i);
	x.setElementAt(tmp, j);
    }
} // end of class Sorter


